/**
 * Visualizador da simulao.
 * 
 * @author Flavio Regis de Arruda.
 * @version 1.0
 */
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.Math.*;
import java.util.zip.*;
import java.util.*;

public class Computer extends SimulatorComponent{

    String msg;
    private final Color BACKGROUND = Color.white;
    private final Color FOREGROUND = Color.black;
    private int width;
    private int height;

    private Processor  processor;
    private Process    process;
    private Queue queue;
    private Clock clock;
    private Cronometer crono;
    private int pulses=0;

    public static int ARRIVAL=5;
    public static int SERVICE=10;
    public static Network net= new Network();
    static int ID=0;
    private int CID; 
    public Computer(){
	msg = "";
	setBackground(BACKGROUND);
	queue = new Queue();
	crono = new Cronometer();
	net.add(this);
	CID=++ID;

    }

    public Computer(String msg){
	
	this.msg = msg;
	//process = new Process();
	queue = new Queue();
	crono = new Cronometer();
	net.add(this);
	CID=++ID;
	if(CID==2){
	    queue.insert(new Process(SERVICE));
	    queue.insert(new Process(SERVICE));
		}
    }


    public static void setArrival(int a){
	ARRIVAL = a;
    }

    public static void setService(int s){
	SERVICE = s;
    }

    public static void setQuantum(int q){
	    Processor.setQuantum(q);
    }

    public void setProcessor(Processor p){
	processor = p;
	processor.setQueue(queue);
    }

    public void setProcess(Process pr){
	process = pr;
    }

    public void setQueue(Queue q){
	queue = q;
    }

    public void setClock(Clock c){
	clock = c;
    }

    public boolean isUnder(){
	return (!queue.isFull());
    }


    public void pulse(){
	System.out.println("pulse");
	pulses++;
	Graphics g = getGraphics();
	if(g==null){
	    System.out.println("Computer ERROR: g EH NULL!!!");
	    return;
	}else{
	    processor.setGraphics(g);
	    queue.setGraphics(g);
	    crono.setGraphics(g);

	}
	//Caso normal.
	if(pulses >= ARRIVAL && queue.isFull()==false && process==null){
	    System.out.println("Computer->Process created!");
	    queue.insert(new Process(SERVICE));
	    queue.paint(g);
	    pulses=0;
	}else //Se esse computador recebeu um processo de outro computador.
	    if(pulses >= ARRIVAL && process != null && queue.isFull()==false){
		System.out.println("Computer "+CID+"->Colocando processo "+process.getPid()+" migrado na fila");
		queue.insert(process);
		queue.paint(g);
		pulses=0;
		process=null;
	    }else //Tenta mandar processo para outra mquina.
		if(pulses >= ARRIVAL && queue.isFull()==true){
		    Process novo = new Process(SERVICE);
		    Computer dest = net.getUnder();
		    if (dest!=null && dest!=this){
			System.out.println("Computer "+CID+"->Migrando processo com ID "+novo.getPid());
			net.sendProcess(novo, dest);
		    }else
			return;
		    pulses=0;
		}
	processor.pulse();
	crono.pulse();
	//repaint();
    }

    public void update(Graphics g){
	System.out.println("Computer update!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*********");
	//paint(g);
    }

    public void paint(Graphics g){

	width = getSize().width;
	height = getSize().height;

	/* Paint de background */
	g.setColor(BACKGROUND);
	g.fillRect((int)Math.floor(0.05*width), (int) Math.floor(0.05*height), (int) Math.floor(0.9*width), (int)Math.floor(0.9*height));
	g.setColor(FOREGROUND);

	/*	
		System.out.println("Computer.X="+getX()+" Computer.Y="+getY());
		System.out.println("Computer.width="+width+" Computer.height="+height);
	*/

	/* Draw all the parts of Computer */
	Color old = g.getColor();
	g.setColor(FOREGROUND);
	g.drawRect((int)Math.floor(0.05*width), (int) Math.floor(0.05*height), (int) Math.floor(0.9*width), (int)Math.floor(0.9*height));
	g.setColor(old);
	// Draw message.
	drawMessage(g, msg);

	// Draw processor.
	if (processor != null){
	    int x = (int) Math.floor(width * 0.2);
	    int y = (int) Math.floor(height * 0.7);
	    processor.setLocation(x , y);
	    processor.setSize(width/3, height/7);
	    processor.setBackground(BACKGROUND);
	    processor.setFont(getFont());

	    processor.paint(g);
	}

	// Draw process.
	//if(process != null){
	//  int x = (int) Math.floor(width * 0.65);
	//  int y = (int) Math.floor(height * 0.7);
	//  process.setLocation(x , y);
	//  process.setSize(80*width/300, 50*height/700);
	//  process.setBackground(BACKGROUND);
	//  process.setFont(getFont());
	//  process.paint(g);
	//}

	if(crono!=null){
	    int x = (int) Math.floor(width * 0.75);
	    int y = (int) Math.floor(height * 0.8);
	    crono.setLocation(x , y);
	    crono.setSize(55*width/300, 100*height/700);
	    //crono.setBackground(BACKGROUND);
	    crono.setFont(getFont());
	    crono.paint(g);
	}


	// Draw queue.
	if(queue != null && processor!= null){
	    int x = (int) Math.floor(width * 0.2);
	    int y = (int) Math.floor(height * 0.2);
	    queue.setLocation(x , y);
	    queue.setSize(processor.getSize());
	    queue.setBackground(BACKGROUND);
	    queue.setFont(getFont());
	    queue.paint(g);

	}

    }

    private void drawMessage(Graphics g, String msg){
	FontMetrics fm = getFontMetrics(getFont());
	g.drawString(msg, (width - fm.stringWidth(msg))/2, (int)Math.floor(0.9*height));      
    }


}//end of class Computer.

